@@ -16,7 +16,7 @@ public class TlsServerProtocol
     extends TlsProtocol
 {
     protected TlsServer tlsServer = null;
-    protected TlsServerContextImpl tlsServerContext = null;
+    TlsServerContextImpl tlsServerContext = null;
 
     protected TlsKeyExchange keyExchange = null;
     protected TlsCredentials serverCredentials = null;
@@ -25,16 +25,46 @@ public class TlsServerProtocol
     protected short clientCertificateType = -1;
     protected TlsHandshakeHash prepareFinishHash = null;
 
+    /**
+     * Constructor for blocking mode.
+     * @param input The stream of data from the client
+     * @param output The stream of data to the client
+     * @param secureRandom Random number generator for various cryptographic functions
+     */
     public TlsServerProtocol(InputStream input, OutputStream output, SecureRandom secureRandom)
     {
         super(input, output, secureRandom);
     }
 
     /**
-     * Receives a TLS handshake in the role of server
+     * Constructor for non-blocking mode.<br>
+     * <br>
+     * When data is received, use {@link #offerInput(java.nio.ByteBuffer)} to
+     * provide the received ciphertext, then use
+     * {@link #readInput(byte[], int, int)} to read the corresponding cleartext.<br>
+     * <br>
+     * Similarly, when data needs to be sent, use
+     * {@link #offerOutput(byte[], int, int)} to provide the cleartext, then use
+     * {@link #readOutput(byte[], int, int)} to get the corresponding
+     * ciphertext.
+     * 
+     * @param secureRandom
+     *            Random number generator for various cryptographic functions
+     */
+    public TlsServerProtocol(SecureRandom secureRandom)
+    {
+        super(secureRandom);
+    }
+
+    /**
+     * Receives a TLS handshake in the role of server.<br>
+     * <br>
+     * In blocking mode, this will not return until the handshake is complete.
+     * In non-blocking mode, use {@link TlsPeer#notifyHandshakeComplete()} to
+     * receive a callback when the handshake is complete.
      *
      * @param tlsServer
-     * @throws IOException If handshake was not successful.
+     * @throws IOException If in blocking mode and handshake was not successful.
      */
     public void accept(TlsServer tlsServer)
         throws IOException
@@ -63,7 +93,7 @@ public class TlsServerProtocol
 
         this.recordStream.setRestrictReadVersion(false);
 
-        completeHandshake();
+        blockForHandshake();
     }
 
     protected void cleanupHandshake()
@@ -76,7 +106,12 @@ public class TlsServerProtocol
         this.prepareFinishHash = null;
     }
 
-    protected AbstractTlsContext getContext()
+    protected TlsContext getContext()
+    {
+        return tlsServerContext;
+    }
+
+    AbstractTlsContext getContextAdmin()
     {
         return tlsServerContext;
     }
@@ -86,11 +121,9 @@ public class TlsServerProtocol
         return tlsServer;
     }
 
-    protected void handleHandshakeMessage(short type, byte[] data)
+    protected void handleHandshakeMessage(short type, ByteArrayInputStream buf)
         throws IOException
     {
-        ByteArrayInputStream buf = new ByteArrayInputStream(data);
-
         switch (type)
         {
         case HandshakeType.client_hello:
@@ -105,6 +138,8 @@ public class TlsServerProtocol
                 sendServerHelloMessage();
                 this.connection_state = CS_SERVER_HELLO;
 
+                recordStream.notifyHelloComplete();
+
                 Vector serverSupplementalData = tlsServer.getServerSupplementalData();
                 if (serverSupplementalData != null)
                 {
@@ -161,6 +196,11 @@ public class TlsServerProtocol
                     this.certificateRequest = tlsServer.getCertificateRequest();
                     if (this.certificateRequest != null)
                     {
+                        if (TlsUtils.isTLSv12(getContext()) != (certificateRequest.getSupportedSignatureAlgorithms() != null))
+                        {
+                            throw new TlsFatalAlert(AlertDescription.internal_error);
+                        }
+
                         this.keyExchange.validateCertificateRequest(certificateRequest);
 
                         sendCertificateRequestMessage(certificateRequest);
@@ -178,6 +218,11 @@ public class TlsServerProtocol
 
                 break;
             }
+            case CS_END:
+            {
+                refuseRenegotiation();
+                break;
+            }
             default:
                 throw new TlsFatalAlert(AlertDescription.unexpected_message);
             }
@@ -326,7 +371,8 @@ public class TlsServerProtocol
 
                 sendFinishedMessage();
                 this.connection_state = CS_SERVER_FINISHED;
-                this.connection_state = CS_END;
+
+                completeHandshake();
                 break;
             }
             default:
@@ -366,6 +412,7 @@ public class TlsServerProtocol
         default:
         {
             super.handleWarningMessage(description);
+            break;
         }
         }
     }
@@ -378,7 +425,7 @@ public class TlsServerProtocol
             throw new IllegalStateException();
         }
 
-        if (this.peerCertificate != null)
+        if (peerCertificate != null)
         {
             throw new TlsFatalAlert(AlertDescription.unexpected_message);
         }
@@ -428,40 +475,49 @@ public class TlsServerProtocol
     protected void receiveCertificateVerifyMessage(ByteArrayInputStream buf)
         throws IOException
     {
+        if (certificateRequest == null)
+        {
+            throw new IllegalStateException();
+        }
+
         DigitallySigned clientCertificateVerify = DigitallySigned.parse(getContext(), buf);
 
         assertEmpty(buf);
 
         // Verify the CertificateVerify message contains a correct signature.
-        boolean verified = false;
         try
         {
-            byte[] certificateVerifyHash;
+            SignatureAndHashAlgorithm signatureAlgorithm = clientCertificateVerify.getAlgorithm();
+
+            byte[] hash;
             if (TlsUtils.isTLSv12(getContext()))
             {
-                certificateVerifyHash = prepareFinishHash.getFinalHash(clientCertificateVerify.getAlgorithm().getHash());
+                TlsUtils.verifySupportedSignatureAlgorithm(certificateRequest.getSupportedSignatureAlgorithms(), signatureAlgorithm);
+                hash = prepareFinishHash.getFinalHash(signatureAlgorithm.getHash());
             }
             else
             {
-                certificateVerifyHash = TlsProtocol.getCurrentPRFHash(getContext(), prepareFinishHash, null);
+                hash = securityParameters.getSessionHash();
             }
 
-            org.bouncycastle.asn1.x509.Certificate x509Cert = this.peerCertificate.getCertificateAt(0);
+            org.bouncycastle.asn1.x509.Certificate x509Cert = peerCertificate.getCertificateAt(0);
             SubjectPublicKeyInfo keyInfo = x509Cert.getSubjectPublicKeyInfo();
             AsymmetricKeyParameter publicKey = PublicKeyFactory.createKey(keyInfo);
 
-            TlsSigner tlsSigner = TlsUtils.createTlsSigner(this.clientCertificateType);
+            TlsSigner tlsSigner = TlsUtils.createTlsSigner(clientCertificateType);
             tlsSigner.init(getContext());
-            verified = tlsSigner.verifyRawSignature(clientCertificateVerify.getAlgorithm(),
-                clientCertificateVerify.getSignature(), publicKey, certificateVerifyHash);
+            if (!tlsSigner.verifyRawSignature(signatureAlgorithm, clientCertificateVerify.getSignature(), publicKey, hash))
+            {
+                throw new TlsFatalAlert(AlertDescription.decrypt_error);
+            }
+        }
+        catch (TlsFatalAlert e)
+        {
+            throw e;
         }
         catch (Exception e)
         {
-        }
-
-        if (!verified)
-        {
-            throw new TlsFatalAlert(AlertDescription.decrypt_error);
+            throw new TlsFatalAlert(AlertDescription.decrypt_error, e);
         }
     }
 
@@ -469,6 +525,8 @@ public class TlsServerProtocol
         throws IOException
     {
         ProtocolVersion client_version = TlsUtils.readVersion(buf);
+        recordStream.setWriteVersion(client_version);
+
         if (client_version.isDTLS())
         {
             throw new TlsFatalAlert(AlertDescription.illegal_parameter);
@@ -516,9 +574,18 @@ public class TlsServerProtocol
          */
         this.clientExtensions = readExtensions(buf);
 
-        getContext().setClientVersion(client_version);
+        /*
+         * TODO[session-hash]
+         * 
+         * draft-ietf-tls-session-hash-04 4. Clients and servers SHOULD NOT accept handshakes
+         * that do not use the extended master secret [..]. (and see 5.2, 5.3)
+         */
+        this.securityParameters.extendedMasterSecret = TlsExtensionsUtils.hasExtendedMasterSecretExtension(clientExtensions);
+
+        getContextAdmin().setClientVersion(client_version);
 
         tlsServer.notifyClientVersion(client_version);
+        tlsServer.notifyFallback(Arrays.contains(offeredCipherSuites, CipherSuite.TLS_FALLBACK_SCSV));
 
         securityParameters.clientRandom = client_random;
 
@@ -570,6 +637,9 @@ public class TlsServerProtocol
 
         if (clientExtensions != null)
         {
+            // NOTE: Validates the padding extension data, if present
+            TlsExtensionsUtils.getPaddingExtension(clientExtensions);
+
             tlsServer.processClientExtensions(clientExtensions);
         }
     }
@@ -577,14 +647,24 @@ public class TlsServerProtocol
     protected void receiveClientKeyExchangeMessage(ByteArrayInputStream buf)
         throws IOException
     {
-        this.keyExchange.processClientKeyExchange(buf);
+        keyExchange.processClientKeyExchange(buf);
 
         assertEmpty(buf);
 
-        establishMasterSecret(getContext(), keyExchange);
-        recordStream.setPendingConnectionState(getPeer().getCompression(), getPeer().getCipher());
+        if (TlsUtils.isSSL(getContext()))
+        {
+            establishMasterSecret(getContext(), keyExchange);
+        }
 
         this.prepareFinishHash = recordStream.prepareToFinish();
+        this.securityParameters.sessionHash = getCurrentPRFHash(getContext(), prepareFinishHash, null);
+
+        if (!TlsUtils.isSSL(getContext()))
+        {
+            establishMasterSecret(getContext(), keyExchange);
+        }
+
+        recordStream.setPendingConnectionState(getPeer().getCompression(), getPeer().getCipher());
 
         if (!expectSessionTicket)
         {
@@ -632,19 +712,21 @@ public class TlsServerProtocol
     {
         HandshakeMessage message = new HandshakeMessage(HandshakeType.server_hello);
 
-        ProtocolVersion server_version = tlsServer.getServerVersion();
-        if (!server_version.isEqualOrEarlierVersionOf(getContext().getClientVersion()))
         {
-            throw new TlsFatalAlert(AlertDescription.internal_error);
+            ProtocolVersion server_version = tlsServer.getServerVersion();
+            if (!server_version.isEqualOrEarlierVersionOf(getContext().getClientVersion()))
+            {
+                throw new TlsFatalAlert(AlertDescription.internal_error);
+            }
+    
+            recordStream.setReadVersion(server_version);
+            recordStream.setWriteVersion(server_version);
+            recordStream.setRestrictReadVersion(true);
+            getContextAdmin().setServerVersion(server_version);
+    
+            TlsUtils.writeVersion(server_version, message);
         }
 
-        recordStream.setReadVersion(server_version);
-        recordStream.setWriteVersion(server_version);
-        recordStream.setRestrictReadVersion(true);
-        getContext().setServerVersion(server_version);
-
-        TlsUtils.writeVersion(server_version, message);
-
         message.write(this.securityParameters.serverRandom);
 
         /*
@@ -654,17 +736,17 @@ public class TlsServerProtocol
         TlsUtils.writeOpaque8(TlsUtils.EMPTY_BYTES, message);
 
         int selectedCipherSuite = tlsServer.getSelectedCipherSuite();
-        if (!Arrays.contains(this.offeredCipherSuites, selectedCipherSuite)
+        if (!Arrays.contains(offeredCipherSuites, selectedCipherSuite)
             || selectedCipherSuite == CipherSuite.TLS_NULL_WITH_NULL_NULL
-            || selectedCipherSuite == CipherSuite.TLS_EMPTY_RENEGOTIATION_INFO_SCSV
-            || !TlsUtils.isValidCipherSuiteForVersion(selectedCipherSuite, server_version))
+            || CipherSuite.isSCSV(selectedCipherSuite)
+            || !TlsUtils.isValidCipherSuiteForVersion(selectedCipherSuite, getContext().getServerVersion()))
         {
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
         securityParameters.cipherSuite = selectedCipherSuite;
 
         short selectedCompressionMethod = tlsServer.getSelectedCompressionMethod();
-        if (!Arrays.contains(this.offeredCompressionMethods, selectedCompressionMethod))
+        if (!Arrays.contains(offeredCompressionMethods, selectedCompressionMethod))
         {
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
@@ -697,11 +779,17 @@ public class TlsServerProtocol
                  * If the secure_renegotiation flag is set to TRUE, the server MUST include an empty
                  * "renegotiation_info" extension in the ServerHello message.
                  */
-                this.serverExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(this.serverExtensions);
+                this.serverExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(serverExtensions);
                 this.serverExtensions.put(EXT_RenegotiationInfo, createRenegotiationInfo(TlsUtils.EMPTY_BYTES));
             }
         }
 
+        if (securityParameters.extendedMasterSecret)
+        {
+            this.serverExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(serverExtensions);
+            TlsExtensionsUtils.addExtendedMasterSecretExtension(serverExtensions);
+        }
+
         /*
          * TODO RFC 3546 2.3 If [...] the older session is resumed, then the server MUST ignore
          * extensions appearing in the client hello, and send a server hello containing no
@@ -710,45 +798,39 @@ public class TlsServerProtocol
 
         if (this.serverExtensions != null)
         {
-            this.securityParameters.encryptThenMAC = TlsExtensionsUtils.hasEncryptThenMACExtension(this.serverExtensions);
+            this.securityParameters.encryptThenMAC = TlsExtensionsUtils.hasEncryptThenMACExtension(serverExtensions);
 
             this.securityParameters.maxFragmentLength = processMaxFragmentLengthExtension(clientExtensions,
-                this.serverExtensions, AlertDescription.internal_error);
+                serverExtensions, AlertDescription.internal_error);
 
-            this.securityParameters.truncatedHMac = TlsExtensionsUtils.hasTruncatedHMacExtension(this.serverExtensions);
+            this.securityParameters.truncatedHMac = TlsExtensionsUtils.hasTruncatedHMacExtension(serverExtensions);
 
             /*
              * TODO It's surprising that there's no provision to allow a 'fresh' CertificateStatus to be sent in
              * a session resumption handshake.
              */
-            this.allowCertificateStatus = !this.resumedSession
-                && TlsUtils.hasExpectedEmptyExtensionData(this.serverExtensions, TlsExtensionsUtils.EXT_status_request,
+            this.allowCertificateStatus = !resumedSession
+                && TlsUtils.hasExpectedEmptyExtensionData(serverExtensions, TlsExtensionsUtils.EXT_status_request,
                     AlertDescription.internal_error);
 
-            this.expectSessionTicket = !this.resumedSession
-                && TlsUtils.hasExpectedEmptyExtensionData(this.serverExtensions, TlsProtocol.EXT_SessionTicket,
+            this.expectSessionTicket = !resumedSession
+                && TlsUtils.hasExpectedEmptyExtensionData(serverExtensions, TlsProtocol.EXT_SessionTicket,
                     AlertDescription.internal_error);
 
-            writeExtensions(message, this.serverExtensions);
-        }
-
-        if (this.securityParameters.maxFragmentLength >= 0)
-        {
-            int plainTextLimit = 1 << (8 + this.securityParameters.maxFragmentLength);
-            recordStream.setPlaintextLimit(plainTextLimit);
+            writeExtensions(message, serverExtensions);
         }
 
         securityParameters.prfAlgorithm = getPRFAlgorithm(getContext(), securityParameters.getCipherSuite());
 
         /*
-         * RFC 5264 7.4.9. Any cipher suite which does not explicitly specify verify_data_length has
+         * RFC 5246 7.4.9. Any cipher suite which does not explicitly specify verify_data_length has
          * a verify_data_length equal to 12. This includes all existing cipher suites.
          */
         securityParameters.verifyDataLength = 12;
 
-        message.writeToRecordStream();
+        applyMaxFragmentLengthExtension();
 
-        this.recordStream.notifyHelloComplete();
+        message.writeToRecordStream();
     }
 
     protected void sendServerHelloDoneMessage()
@@ -773,6 +855,6 @@ public class TlsServerProtocol
 
     protected boolean expectCertificateVerifyMessage()
     {
-        return this.clientCertificateType >= 0 && TlsUtils.hasSigningCapability(this.clientCertificateType);
+        return clientCertificateType >= 0 && TlsUtils.hasSigningCapability(clientCertificateType);
     }
 }
