@@ -322,7 +322,27 @@ public class PGPSecretKey
     {
         return pub.getKeyID();
     }
-    
+
+    /**
+     * Return the S2K usage associated with this key.
+     *
+     * @return the key's S2K usage
+     */
+    public int getS2KUsage()
+    {
+        return secret.getS2KUsage();
+    }
+
+    /**
+     * Return the S2K used to process this key
+     *
+     * @return the key's S2K, null if one is not present.
+     */
+    public S2K getS2K()
+    {
+        return secret.getS2K();
+    }
+
     /**
      * Return the public key associated with this key.
      * 
@@ -672,6 +692,10 @@ public class PGPSecretKey
         }
         else
         {
+            if (s2kUsage == SecretKeyPacket.USAGE_NONE)
+            {
+                s2kUsage = SecretKeyPacket.USAGE_CHECKSUM;
+            }
             if (key.secret.getPublicKeyPacket().getVersion() < 4)
             {
                 // Version 2 or 3 - RSA Keys only
