@@ -8,6 +8,7 @@ import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Vector;
 
+import org.bouncycastle.tls.crypto.TlsStreamSigner;
 import org.bouncycastle.util.Arrays;
 
 public class TlsClientProtocol
@@ -127,11 +128,9 @@ public class TlsClientProtocol
         return tlsClient;
     }
 
-    protected void handleHandshakeMessage(short type, byte[] data)
+    protected void handleHandshakeMessage(short type, ByteArrayInputStream buf)
         throws IOException
     {
-        ByteArrayInputStream buf = new ByteArrayInputStream(data);
-
         if (this.resumedSession)
         {
             if (type != HandshakeType.finished || this.connection_state != CS_SERVER_HELLO)
@@ -144,7 +143,6 @@ public class TlsClientProtocol
 
             sendFinishedMessage();
             this.connection_state = CS_CLIENT_FINISHED;
-            this.connection_state = CS_END;
 
             completeHandshake();
             return;
@@ -240,7 +238,6 @@ public class TlsClientProtocol
             {
                 processFinishedMessage(buf);
                 this.connection_state = CS_SERVER_FINISHED;
-                this.connection_state = CS_END;
 
                 completeHandshake();
                 break;
@@ -331,8 +328,6 @@ public class TlsClientProtocol
 
                 this.connection_state = CS_SERVER_HELLO_DONE;
 
-                this.recordStream.getHandshakeHash().sealHashAlgorithms();
-
                 Vector clientSupplementalData = tlsClient.getClientSupplementalData();
                 if (clientSupplementalData != null)
                 {
@@ -340,16 +335,19 @@ public class TlsClientProtocol
                 }
                 this.connection_state = CS_CLIENT_SUPPLEMENTAL_DATA;
 
-                TlsCredentials clientCreds = null;
+                TlsCredentials clientCredentials = null;
+                TlsCredentialedSigner credentialedSigner = null;
+                TlsStreamSigner streamSigner = null;
+
                 if (certificateRequest == null)
                 {
                     this.keyExchange.skipClientCredentials();
                 }
                 else
                 {
-                    clientCreds = validateCredentials(this.authentication.getClientCredentials(certificateRequest));
+                    clientCredentials = validateCredentials(this.authentication.getClientCredentials(certificateRequest));
 
-                    if (clientCreds == null)
+                    if (clientCredentials == null)
                     {
                         this.keyExchange.skipClientCredentials();
 
@@ -363,14 +361,23 @@ public class TlsClientProtocol
                     }
                     else
                     {
-                        this.keyExchange.processClientCredentials(clientCreds);
+                        this.keyExchange.processClientCredentials(clientCredentials);
 
-                        sendCertificateMessage(clientCreds.getCertificate());
+                        sendCertificateMessage(clientCredentials.getCertificate());
+
+                        if (clientCredentials instanceof TlsCredentialedSigner)
+                        {
+                            credentialedSigner = (TlsCredentialedSigner)clientCredentials;
+                            streamSigner = credentialedSigner.getStreamSigner();
+                        }
                     }
                 }
 
                 this.connection_state = CS_CLIENT_CERTIFICATE;
 
+                boolean forceBuffering = streamSigner != null;
+                TlsUtils.sealHandshakeHash(getContext(), this.recordStream.getHandshakeHash(), forceBuffering);
+
                 /*
                  * Send the client key exchange message, depending on the key exchange we are using
                  * in our CipherSuite.
@@ -384,7 +391,7 @@ public class TlsClientProtocol
                 }
 
                 TlsHandshakeHash prepareFinishHash = recordStream.prepareToFinish();
-                this.securityParameters.sessionHash = getCurrentPRFHash(getContext(), prepareFinishHash, null);
+                this.securityParameters.sessionHash = TlsUtils.getCurrentPRFHash(prepareFinishHash);
 
                 if (!TlsUtils.isSSL(getContext()))
                 {
@@ -393,30 +400,11 @@ public class TlsClientProtocol
 
                 recordStream.setPendingConnectionState(getPeer().getCompression(), getPeer().getCipher());
 
-                if (clientCreds != null && clientCreds instanceof TlsCredentialedSigner)
+                if (credentialedSigner != null)
                 {
-                    TlsCredentialedSigner signerCredentials = (TlsCredentialedSigner)clientCreds;
-
-                    /*
-                     * RFC 5246 4.7. digitally-signed element needs SignatureAndHashAlgorithm from TLS 1.2
-                     */
-                    SignatureAndHashAlgorithm signatureAndHashAlgorithm = TlsUtils.getSignatureAndHashAlgorithm(
-                        getContext(), signerCredentials);
-
-                    byte[] hash;
-                    if (signatureAndHashAlgorithm == null)
-                    {
-                        hash = securityParameters.getSessionHash();
-                    }
-                    else
-                    {
-                        hash = prepareFinishHash.getFinalHash(signatureAndHashAlgorithm.getHash());
-                    }
-
-                    byte[] signature = signerCredentials.generateRawSignature(hash);
-                    DigitallySigned certificateVerify = new DigitallySigned(signatureAndHashAlgorithm, signature);
+                    DigitallySigned certificateVerify = TlsUtils.generateCertificateVerify(getContext(),
+                        credentialedSigner, streamSigner, prepareFinishHash);
                     sendCertificateVerifyMessage(certificateVerify);
-
                     this.connection_state = CS_CERTIFICATE_VERIFY;
                 }
 
@@ -425,7 +413,7 @@ public class TlsClientProtocol
                 break;
             }
             default:
-                throw new TlsFatalAlert(AlertDescription.handshake_failure);
+                throw new TlsFatalAlert(AlertDescription.unexpected_message);
             }
 
             this.connection_state = CS_CLIENT_FINISHED;
@@ -811,7 +799,7 @@ public class TlsClientProtocol
             this.securityParameters.getCipherSuite());
 
         /*
-         * RFC 5264 7.4.9. Any cipher suite which does not explicitly specify
+         * RFC 5246 7.4.9. Any cipher suite which does not explicitly specify
          * verify_data_length has a verify_data_length equal to 12. This includes all
          * existing cipher suites.
          */
