@@ -24,6 +24,7 @@ import org.bouncycastle.crypto.digests.SHA384Digest;
 import org.bouncycastle.crypto.digests.SHA512Digest;
 import org.bouncycastle.crypto.encodings.PKCS1Encoding;
 import org.bouncycastle.crypto.engines.AESEngine;
+import org.bouncycastle.crypto.engines.ARIAEngine;
 import org.bouncycastle.crypto.engines.CamelliaEngine;
 import org.bouncycastle.crypto.engines.ChaCha7539Engine;
 import org.bouncycastle.crypto.engines.DESedeEngine;
@@ -127,7 +128,6 @@ public class BcTlsCrypto
         return nonce;
     }
 
-
     public SecureRandom getSecureRandom()
     {
         return entropySource;
@@ -174,6 +174,16 @@ public class BcTlsCrypto
         case EncryptionAlgorithm.AES_256_OCB_TAGLEN96:
             // NOTE: Ignores macAlgorithm
             return createCipher_AES_OCB(cryptoParams, 32, 12);
+        case EncryptionAlgorithm.ARIA_128_CBC:
+            return createARIACipher(cryptoParams, 16, macAlgorithm);
+        case EncryptionAlgorithm.ARIA_128_GCM:
+            // NOTE: Ignores macAlgorithm
+            return createCipher_ARIA_GCM(cryptoParams, 16, 16);
+        case EncryptionAlgorithm.ARIA_256_CBC:
+            return createARIACipher(cryptoParams, 32, macAlgorithm);
+        case EncryptionAlgorithm.ARIA_256_GCM:
+            // NOTE: Ignores macAlgorithm
+            return createCipher_ARIA_GCM(cryptoParams, 32, 16);
         case EncryptionAlgorithm.CAMELLIA_128_CBC:
             return createCamelliaCipher(cryptoParams, 16, macAlgorithm);
         case EncryptionAlgorithm.CAMELLIA_128_GCM:
@@ -238,6 +248,21 @@ public class BcTlsCrypto
         };
     }
 
+    public boolean hasAllRawSignatureAlgorithms()
+    {
+        return true;
+    }
+
+    public boolean hasDHAgreement()
+    {
+        return true;
+    }
+
+    public boolean hasECDHAgreement()
+    {
+        return true;
+    }
+
     public boolean hasEncryptionAlgorithm(int encryptionAlgorithm)
     {
         return true;
@@ -253,12 +278,28 @@ public class BcTlsCrypto
         return true;
     }
 
+    public boolean hasNamedCurve(int curveID)
+    {
+        // TODO: at the moment we support everything
+        return true;
+    }
+
+    public boolean hasRSAEncryption()
+    {
+        return true;
+    }
+
+    public boolean hasSignatureAlgorithm(int signatureAlgorithm)
+    {
+        return true;
+    }
+
     public boolean hasSignatureAndHashAlgorithm(SignatureAndHashAlgorithm sigAndHashAlgorithm)
     {
         return true;
     }
 
-    public boolean hasRSAEncryption()
+    public boolean hasSRPAuthentication()
     {
         return true;
     }
@@ -414,6 +455,13 @@ public class BcTlsCrypto
             createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm), cipherKeySize);
     }
 
+    protected TlsCipher createARIACipher(TlsCryptoParameters cryptoParams, int cipherKeySize, int macAlgorithm)
+        throws IOException
+    {
+        return new TlsBlockCipher(this, cryptoParams, new BlockOperator(createARIABlockCipher(), true), new BlockOperator(createARIABlockCipher(), false),
+            createMac(cryptoParams, macAlgorithm), createMac(cryptoParams, macAlgorithm), cipherKeySize);
+    }
+
     protected TlsCipher createCamelliaCipher(TlsCryptoParameters cryptoParams, int cipherKeySize, int macAlgorithm)
         throws IOException
     {
@@ -450,6 +498,13 @@ public class BcTlsCrypto
             cipherKeySize, macSize, TlsAEADCipher.NONCE_RFC7905);
     }
 
+    protected TlsAEADCipher createCipher_ARIA_GCM(TlsCryptoParameters cryptoParams, int cipherKeySize, int macSize)
+        throws IOException
+    {
+        return new TlsAEADCipher(cryptoParams, new AeadOperator(createAEADBlockCipher_ARIA_GCM(), true), new AeadOperator(createAEADBlockCipher_ARIA_GCM(), false),
+            cipherKeySize, macSize);
+    }
+
     protected TlsAEADCipher createCipher_Camellia_GCM(TlsCryptoParameters cryptoParams, int cipherKeySize, int macSize)
         throws IOException
     {
@@ -489,6 +544,11 @@ public class BcTlsCrypto
         return new AESEngine();
     }
 
+    protected BlockCipher createARIAEngine()
+    {
+        return new ARIAEngine();
+    }
+
     protected BlockCipher createCamelliaEngine()
     {
         return new CamelliaEngine();
@@ -499,6 +559,11 @@ public class BcTlsCrypto
         return new CBCBlockCipher(createAESEngine());
     }
 
+    protected BlockCipher createARIABlockCipher()
+    {
+        return new CBCBlockCipher(createARIAEngine());
+    }
+
     protected AEADBlockCipher createAEADBlockCipher_AES_CCM()
     {
         return new CCMBlockCipher(createAESEngine());
@@ -515,6 +580,12 @@ public class BcTlsCrypto
         return new OCBBlockCipher(createAESEngine(), createAESEngine());
     }
 
+    protected AEADBlockCipher createAEADBlockCipher_ARIA_GCM()
+    {
+        // TODO Consider allowing custom configuration of multiplier
+        return new GCMBlockCipher(createARIAEngine());
+    }
+
     protected AEADBlockCipher createAEADBlockCipher_Camellia_GCM()
     {
         // TODO Consider allowing custom configuration of multiplier
